import Qt3D.Core 2.0
import Qt3D.Render 2.0
import Qt3D.Extras 2.0
import Qt3D.Input 2.0
import MyQt3D 1.0

Entity {
    id: sceneRoot

    Camera {
        id: camera
        projectionType: CameraLens.PerspectiveProjection
        fieldOfView: 45
        aspectRatio: 16/9
        nearPlane : 0.1
        farPlane : 100.0
        position: Qt.vector3d( 1.0, 1.0, 1.0 )
        upVector: Qt.vector3d( 0.0, 1.0, 0.0 )
        viewCenter: Qt.vector3d( 0.0, 0.0, 0.0 )
    }

    OrbitCameraController {
        id: orbitCameraController
        camera: camera
        linearSpeed: 10.0
    }

    components: [
        RenderSettings {
            pickingSettings.pickMethod: PickingSettings.TrianglePicking
            pickingSettings.pickResultMode: PickingSettings.AllPicks

            activeFrameGraph: ForwardRenderer {
                clearColor: Qt.rgba(0, 0.5, 1, 1)
                camera: camera
            }
        },
        InputSettings { },
        MouseHandler {
            id: mouseHandler
            sourceDevice:  MouseDevice {}
            onReleased: {
                console.log("MouseHandler released");
                if (draggingPlane.draggingTarget !== null) {
                    orbitCameraController.enabled = true;
                    draggingPlane.draggingTarget = null;
                }
            }
        },
        DirectionalLight {}
    ]

    Entity {
        property Transform transform:
            Transform {
                translation: Qt.vector3d(0.0, 0.0, 0.0)
            }

        property var geometry:
           PlaneMesh {
               objectName: "slicePlane"
               width: 1.0
               height: 1.0
           }

       property var material:
           TextureMaterial {
               texture: Texture2D {
                   id: myTexture
                   minificationFilter: Texture.Linear
                   magnificationFilter: Texture.Linear
                   wrapMode {
                       x: WrapMode.Repeat
                       y: WrapMode.Repeat
                   }
                   maximumAnisotropy: 16.0
                   TextureImage {
                       source: "qrc:/Penguins_512.bmp"
                   }
               }
           }

        components: [transform, geometry, material]
    }

    Entity {
        property ObjectPicker picker: ObjectPicker {
            id: draggingPlane
            property var draggingTarget

            dragEnabled: true

            onContainsMouseChanged: {
                console.log("contain: " + containsMouse);
            }
            onPressed: {
                console.log("plane pressed")
            }

            onEntered: {
                console.log("plane entered");
            }
            onClicked: {
                console.log("plane clicked " + pick.worldIntersection);
            }
            onMoved: {
                if (draggingTarget) {
                    console.log("plane moved" + pick.worldIntersection);
                    draggingTarget.transform.translation = pick.worldIntersection;
                }
            }
            onReleased: {
                console.log("plane released");
            }
        }

        property var transform:
            Transform {
                translation: Qt.vector3d(0, 0.2, 0)
            }

        property var geometry:
            PlaneMesh {
                width: 1.0
                height: 1.0
            }

        property var material:
            PhongAlphaMaterial {

                ambient: "gray"
                diffuse: "gray"
                alpha: 0.5
                shininess: 0.0
                specular: "transparent"

                destinationAlphaArg: BlendEquationArguments.One
            }

        components: [transform, geometry, material, picker]
    }

    Entity {
        id: sliceGizmo1

        property bool dragging: draggingPlane.draggingTarget === this

        property ObjectPicker picker: ObjectPicker {
            dragEnabled: true

            onPressed: {
                if (pick.button === PickEvent.LeftButton) {
                    orbitCameraController.enabled = false;
                    console.log("gizmo pressed");
                    pick.setAccepted(false);
                    draggingPlane.draggingTarget = parent;

                }
            }
            onEntered: {
                parent.material.ambient = "red";
                console.log("gizmo entered");
            }
            onExited: {
                parent.material.ambient = "green";
                console.log("gizmo exited");
            }

            onClicked: {
                console.log("gizmo clicked " + pick.worldIntersection);
                orbitCameraController.enabled = true;
                pick.setAccepted(false);
            }
            onMoved: {
                if (parent.dragging == true) {
                    console.log("gizmo moved");
                    pick.setAccepted(false);
                }
            }
            onReleased: {
                console.log("gizmo released")
            }
        }

        property var transform:
            Transform {
                translation: Qt.vector3d(0.2, 0.2, 0.2)
            }

        property var geometry:
            CuboidMesh {
                xExtent: 0.03
                yExtent: 0.03
                zExtent: 0.03
            }

        property var material:
            PhongAlphaMaterial {
                ambient: "green"
                diffuse: ambient
                alpha: 1.0
                shininess: 0.0
            }

        components: [transform, geometry, material, picker]
    }

    Entity {
        id: sliceGizmo2

        property bool dragging: draggingPlane.draggingTarget === this

        property ObjectPicker picker: ObjectPicker {
            hoverEnabled: true

            onPressed: {
                if (pick.button === PickEvent.LeftButton) {
                    orbitCameraController.enabled = false;
                    console.log("gizmo pressed");
                    pick.setAccepted(false);
                    parent.dragging = true;
                    draggingPlane.draggingTarget = parent;
                }
            }
            onEntered: {
                parent.material.ambient = "red";
                console.log("gizmo entered");
            }
            onExited: {
                parent.material.ambient = "blue";
                console.log("gizmo exited");
            }

            onClicked: {
                console.log("gizmo clicked " + pick.worldIntersection);
                orbitCameraController.enabled = true;
                pick.setAccepted(false);
            }
            onMoved: {
                if (parent.dragging == true) {
                    console.log("gizmo moved");
                    pick.setAccepted(false);
                }
            }
            onReleased: {
                console.log("gizmo released")
            }
        }

        property var transform:
            Transform {
                translation: Qt.vector3d(0.4, 0.2, 0.2)
            }

        property var geometry:
            CuboidMesh {
                xExtent: 0.03
                yExtent: 0.03
                zExtent: 0.03
            }

        property var material:
            PhongAlphaMaterial {
                ambient: "blue"
                diffuse: ambient
                alpha: 1.0
                shininess: 0.0
            }

        components: [transform, geometry, material, picker]
    }
}
